@@ -13,21 +13,27 @@
 #    under the License.
 
 import functools
+import logging
 import queue
 import threading
+import time
 
 from concurrent import futures as _futures
 from concurrent.futures import process as _process
 
+from debtcollector import removals
+
 from futurist import _green
 from futurist import _thread
 from futurist import _utils
 
-
 TimeoutError = _futures.TimeoutError
 CancelledError = _futures.CancelledError
 
 
+LOG = logging.getLogger(__name__)
+
+
 class RejectedSubmission(Exception):
     """Exception raised when a submitted call is rejected (for some reason)."""
 
@@ -36,7 +42,7 @@ class RejectedSubmission(Exception):
 Future = _futures.Future
 
 
-class _Gatherer(object):
+class _Gatherer:
     def __init__(self, submit_func, lock_factory, start_before_submit=False):
         self._submit_func = submit_func
         self._stats_lock = lock_factory()
@@ -101,6 +107,10 @@ class ThreadPoolExecutor(_futures.Executor):
 
     It gathers statistics about the submissions executed for post-analysis...
 
+    Note that this executor never shrinks its thread pool, which will cause
+    the pool to eventually reach its maximum capacity defined by max_workers.
+    Check :py:class:`DynamicThreadPoolExecutor` for an alternative.
+
     See: https://docs.python.org/dev/library/concurrent.futures.html
     """
 
@@ -127,7 +137,7 @@ class ThreadPoolExecutor(_futures.Executor):
         if max_workers is None:
             max_workers = _utils.get_optimal_thread_count()
         if max_workers <= 0:
-            raise ValueError("Max workers must be greater than zero")
+            raise ValueError("max_workers must be greater than zero")
         self._max_workers = max_workers
         self._work_queue = queue.Queue()
         self._shutdown_lock = threading.RLock()
@@ -146,18 +156,44 @@ class ThreadPoolExecutor(_futures.Executor):
         """Accessor to determine if the executor is alive/active."""
         return not self._shutdown
 
+    @property
+    def queue_size(self):
+        """The current size of the queue.
+
+        This value represents the number of tasks that are waiting for a free
+        worker thread.
+        """
+        return self._work_queue.qsize()
+
+    @property
+    def num_workers(self):
+        """The current number of worker threads."""
+        return len(self._workers)
+
+    def get_num_idle_workers(self):
+        """Get the number of currently idle threads.
+
+        A thread is idle if it's waiting for new tasks from the queue.
+
+        This method is required to obtain the shutdown lock it provides
+        an accurate count.
+        """
+        with self._shutdown_lock:
+            return sum(1 for w in self._workers if w.idle)
+
+    def _add_thread(self):
+        w = _thread.ThreadWorker.create_and_register(
+            self, self._work_queue)
+        # Always save it before we start (so that even if we fail
+        # starting it we can correctly join on it).
+        self._workers.append(w)
+        w.start()
+
     def _maybe_spin_up(self):
         """Spin up a worker if needed."""
-        # Do more advanced idle checks and/or reaping of very idle
-        # threads in the future....
         if (not self._workers or
                 len(self._workers) < self._max_workers):
-            w = _thread.ThreadWorker.create_and_register(
-                self, self._work_queue)
-            # Always save it before we start (so that even if we fail
-            # starting it we can correctly join on it).
-            self._workers.append(w)
-            w.start()
+            self._add_thread()
 
     def shutdown(self, wait=True):
         with self._shutdown_lock:
@@ -177,6 +213,9 @@ class ThreadPoolExecutor(_futures.Executor):
 
     def submit(self, fn, *args, **kwargs):
         """Submit some work to be executed (and gather statistics)."""
+        # NOTE(dtantsur): DynamicThreadPoolExecutor relies on this lock for
+        # its complex logic around thread management. If you ever decide to
+        # remove it, please add a lock there instead.
         with self._shutdown_lock:
             if self._shutdown:
                 raise RuntimeError('Can not schedule new futures'
@@ -185,6 +224,174 @@ class ThreadPoolExecutor(_futures.Executor):
             return self._gatherer.submit(fn, *args, **kwargs)
 
 
+class DynamicThreadPoolExecutor(ThreadPoolExecutor):
+    """Executor that creates or removes threads on demand.
+
+    As new work is scheduled on the executor, it will try to keep the
+    proportion of busy threads within the provided range (between 40% and 80%
+    by default). A busy thread is a thread that is not waiting on the
+    task queue.
+
+    Each time a task is submitted, the executor makes a decision whether to
+    grow or shrink the pool. It takes the proportion of the number of busy
+    threads to the total number of threads and compares it to shrink_threshold
+    and grow_threshold.
+
+    Initially, the pool is empty, so submitting a task always result in one
+    new thread. Since min_workers must be greater than zero, at least one
+    thread will always be available after this point.
+
+    Once the proportion of busy threads reaches grow_threshold (e.g. 4 out of 5
+    with the default grow_threshold of 0.8), a new thread is created when a
+    task is submitted. If on submitting a task a proportion of busy threads is
+    below shrink_threshold (e.g. only 2 out of 5), one idle thread is stopped.
+
+    The values of grow_threshold and shrink_threshold are different to prevent
+    the number of threads from oscilating on reaching grow_threshold.
+
+    If threads are not created often in your application, the number of idle
+    threads may stay high for a long time. To avoid it, you can call
+    :py:meth:`.maintain` periodically to keep the number of threads within
+    the thresholds.
+
+    """
+
+    def __init__(self, max_workers=None, check_and_reject=None,
+                 min_workers=1, grow_threshold=0.8,
+                 shrink_threshold=0.4):
+        """Initializes a thread pool executor.
+
+        :param max_workers: maximum number of workers that can be
+                            simultaneously active at the same time, further
+                            submitted work will be queued up when this limit
+                            is reached.
+        :type max_workers: int
+        :param check_and_reject: a callback function that will be provided
+                                 two position arguments, the first argument
+                                 will be this executor instance, and the second
+                                 will be the number of currently queued work
+                                 items in this executors backlog; the callback
+                                 should raise a :py:class:`.RejectedSubmission`
+                                 exception if it wants to have this submission
+                                 rejected.
+        :type check_and_reject: callback
+        :param min_workers: the minimum number of workers that can be reached
+                            when shrinking the pool. Note that the pool always
+                            starts at zero workers and will be smaller than
+                            min_workers until enough workers are created.
+                            At least one thread is required.
+        :type max_workers: int
+        :param grow_threshold: minimum proportion of busy threads to total
+                               threads for the pool to grow.
+        :type grow_threshold: float
+        :param shrink_threshold: maximum proportion of busy threads to total
+                                 threads for the pool to shrink.
+        :type shrink_threshold: float
+        """
+        super().__init__(max_workers=max_workers,
+                         check_and_reject=check_and_reject)
+        if min_workers <= 0:
+            raise ValueError('min_workers must be greater than zero')
+        if max_workers and min_workers >= max_workers:
+            raise ValueError('min_workers must be less than max_workers')
+        self._min_workers = min_workers
+
+        if grow_threshold <= 0 or grow_threshold > 1.0:
+            raise ValueError('grow_threshold must be within (0, 1]')
+        if shrink_threshold < 0 or shrink_threshold >= 1.0:
+            raise ValueError('shrink_threshold must be within [0, 1)')
+        if shrink_threshold >= grow_threshold:
+            raise ValueError(
+                'shrink_threshold must be less than grow_threshold')
+        self._grow_threshold = grow_threshold
+        self._shrink_threshold = shrink_threshold
+
+        self._dead_workers = []
+
+    def _drop_thread(self):
+        new_workers = []
+        idle_worker = None
+        for i, w in enumerate(self._workers):
+            if w.idle:
+                new_workers = self._workers[i + 1:]
+                idle_worker = w
+                break
+            new_workers.append(w)
+
+        if idle_worker is None:
+            # Should not actually happen but races are possible; do nothing
+            LOG.warning(
+                'No idle worker thread to delete when shrinking pool %r', self)
+            return False
+
+        w.stop()
+        self._workers = new_workers
+        self._dead_workers.append(w)
+        return True
+
+    def _maybe_spin_up(self):
+        nthreads = self.num_workers
+        if nthreads < self._min_workers:
+            self._add_thread()
+            return True
+
+        # NOTE(dtantsur): here we count the number of threads that are
+        # doing something (i.e. are not waiting on the queue) plus the
+        # number of tasks in the queue. In theory, if there are idle
+        # workers, the queue should be empty. But race conditions are
+        # possible when workers do not pick up tasks quickly enough,
+        # especially in the presence of CPU-bound tasks.
+        idle = self.get_num_idle_workers()
+        busy = (nthreads - idle + self.queue_size) / nthreads
+        if busy >= self._grow_threshold and nthreads < self._max_workers:
+            LOG.debug('Creating a new worker thread for pool %r '
+                      '(%d thread(s) idle, queue size %d, total %d thread(s))',
+                      self, idle, self.queue_size, nthreads)
+            self._add_thread()
+            return True
+        elif busy <= self._shrink_threshold and nthreads > self._min_workers:
+            LOG.debug('Deleting a worker thread from pool %r '
+                      '(%d thread(s) idle, queue size %d, total %d thread(s))',
+                      self, idle, self.queue_size, nthreads)
+            return self._drop_thread()
+
+        return False
+
+    def maintain(self):
+        """Keep the number of threads within the expected range.
+
+        If too many idle threads are running, they are deleted.
+        Additionally, deleted workers are joined to free up resources.
+        """
+        # NOTE(dtantsur): this call can potentially run for some time, so
+        # avoid taking shutdown_lock once and holding it for the entire
+        # duration (blocking any new tasks from being added).
+        keep_going = True
+        while keep_going:
+            if self._shutdown:
+                return
+
+            with self._shutdown_lock:
+                keep_going = self._maybe_spin_up()
+
+            time.sleep(0)
+
+        # NOTE(dtantsur): copy the value of _dead_workers to prevent races with
+        # other invocations for maintain or shutdown.
+        with self._shutdown_lock:
+            dead_workers = self._dead_workers
+            self._dead_workers = []
+
+        for w in dead_workers:
+            w.join()
+
+    def shutdown(self, wait=True):
+        super().shutdown(wait=wait)
+        if wait:
+            for w in self._dead_workers:
+                w.join()
+
+
 class ProcessPoolExecutor(_process.ProcessPoolExecutor):
     """Executor that uses a process pool to execute calls asynchronously.
 
@@ -198,14 +405,14 @@ class ProcessPoolExecutor(_process.ProcessPoolExecutor):
     def __init__(self, max_workers=None):
         if max_workers is None:
             max_workers = _utils.get_optimal_process_count()
-        super(ProcessPoolExecutor, self).__init__(max_workers=max_workers)
+        super().__init__(max_workers=max_workers)
         if self._max_workers <= 0:
             raise ValueError("Max workers must be greater than zero")
         self._gatherer = _Gatherer(
             # Since our submit will use this gatherer we have to reference
             # the parent submit, bound to this instance (which is what we
             # really want to use anyway).
-            super(ProcessPoolExecutor, self).submit,
+            super().submit,
             self.threading.lock_object)
 
     @property
@@ -236,6 +443,10 @@ class SynchronousExecutor(_futures.Executor):
 
     threading = _thread.Threading()
 
+    @removals.removed_kwarg('green',
+                            message="Eventlet support is deprecated. "
+                            "Please migrate your code and stop enforcing "
+                            "its usage.")
     def __init__(self, green=False, run_work_func=lambda work: work.run()):
         """Synchronous executor constructor.
 
@@ -299,11 +510,15 @@ class SynchronousExecutor(_futures.Executor):
         return fut
 
 
+@removals.removed_class("GreenFuture",
+                        message="Eventlet support is deprecated. "
+                        "Please migrate your code and stop using Green "
+                        "future.")
 class GreenFuture(Future):
     __doc__ = Future.__doc__
 
     def __init__(self):
-        super(GreenFuture, self).__init__()
+        super().__init__()
         if not _utils.EVENTLET_AVAILABLE:
             raise RuntimeError('Eventlet is needed to use a green future')
         # NOTE(harlowja): replace the built-in condition with a greenthread
@@ -315,6 +530,10 @@ class GreenFuture(Future):
             self._condition = _green.threading.condition_object()
 
 
+@removals.removed_class("GreenThreadPoolExecutor",
+                        message="Eventlet support is deprecated. "
+                        "Please migrate your code and stop using Green "
+                        "executor.")
 class GreenThreadPoolExecutor(_futures.Executor):
     """Executor that uses a green thread pool to execute calls asynchronously.
 
@@ -415,7 +634,7 @@ class GreenThreadPoolExecutor(_futures.Executor):
             self._pool.waitall()
 
 
-class ExecutorStatistics(object):
+class ExecutorStatistics:
     """Holds *immutable* information about a executors executions."""
 
     __slots__ = ['_failures', '_executed', '_runtime', '_cancelled']
